#ifndef DYN_MODULES_OUTPUT_HH
#define DYN_MODULES_OUTPUT_HH

#include <map>
#include <awali/dyn/core/automaton.hh>
#include <awali/dyn/modules/automaton.hh>
#include <awali/dyn/loading/handler.hh>

namespace awali { namespace dyn {

  std::ostream& daut(automaton_t aut, std::ostream& out);
  std::ostream& dot(automaton_t aut, std::ostream& out, bool history=false,
                                                       bool horizontal=true);
  std::ostream& fado(automaton_t aut, std::ostream& out);
  std::ostream& grail(automaton_t aut, std::ostream& out);
  std::ostream& efsm(automaton_t aut, std::ostream& out);
  std::ostream& json(automaton_t aut, std::ostream& out);

  std::ostream& pdf(automaton_t aut, std::ostream& o, bool history=false, bool horizontal=true);


  struct io {
   private :
    io(std::string name,unsigned v) : name(name), v(v) {
      //instances.push_back(this);
    }
    const std::string name;
    const static std::vector<const io*>& instances();
   public :
    unsigned int v;
    static const io dot;
    static const io fado;
    static const io grail;
    static const io json;
    static const io pdf;
  

    static io of_string(const std::string& str) {
      if (str == "default") 
        return io::json;
      for (const io* x : instances()) {
        if (x->name == str) 
          return *x;
       }
      throw std::runtime_error("Unknown format :"+str);
    };
  };
  const io io::dot("dot",0);
  const io io::fado("fado",1);
  const io io::grail("grail",2);
  const io io::json("json",3);
  const io io::pdf("pdf",4);

  struct autostream {
    private :
      unsigned int v;
      std::ostream& o;
    public :
      autostream(unsigned int v, std::ostream& o) : v(v), o(o) {}
      std::ostream& operator<<(automaton_t aut);
  };

  struct autistream {
    private :
      unsigned int v;
      std::istream& i;
    public :
      autistream(unsigned int v, std::istream& i) : v(v), i(i) {}
      std::istream& operator>>(automaton_t& aut);
  };

  std::ostream& operator<<(std::ostream& o, automaton_t aut);
  autostream operator<<(std::ostream& o, const io& m);
  std::istream& operator>>(std::istream& i, automaton_t& aut);
  autistream operator>>(std::istream& i, const io& m);

  automaton_t fado(std::istream& in);
  automaton_t grail(std::istream& in);



}}//end of ns awali::dyn

#endif
