#ifndef AWALI_ALGOS_IS_DETERMINISTIC_HXX
# define AWALI_ALGOS_IS_DETERMINISTIC_HXX

# include <queue>
# include <unordered_set>

#include <awali/sttc/ctx/traits.hh>

namespace awali { namespace sttc {


  /// Whether state \a s is deterministic in \a aut.
  template <typename Aut>
  inline bool
  is_deterministic(const Aut& aut, state_t s)
  {
    using automaton_t = Aut;
    static_assert(labelset_t_of<automaton_t>::is_free(),
                  "requires free labelset");
    using label_t = label_t_of<automaton_t>;
    std::unordered_set<label_t> seen;
    for (auto t : aut->all_out(s))
      if (!seen.insert(aut->label_of(t)).second)
        return false;
    return true;
  }

  /// Number of non-deterministic states.
  template <class Aut>
  inline size_t
  num_deterministic_states(const Aut& aut)
  {
    static_assert(labelset_t_of<Aut>::is_free(),
                  "requires free labelset");

    size_t res = 0;
    for (auto s: aut->states())
      res += is_deterministic(aut, s);
    return res;
  }

}}//end of ns awali::stc

#endif // !AWALI_ALGOS_IS_DETERMINISTIC_HXX
